
# use the p_load function in pacman package 
# to check if you have installed the packages 
# needed for the workshop
if (!require("pacman")) install.packages("pacman")
pacman::p_load(rvest,magrittr,stringr,stringi,devtools,dplyr,tidyr,XML,RCurl,knitr,effects,msm,ggplot2,ggthemes)

options(stringsAsFactors = FALSE)



# Basic Crawler -----------------------------------------------------------
# Task: collect title, rating, and director of top 250 movies on IMDb
# http://www.imdb.com/chart/top?ref_=nv_ch_250_4

# urls for the 250 movies
link_rvest <- 
read_html("http://www.imdb.com/chart/top?ref_=nv_ch_250_4") %>% 
  html_nodes(".titleColumn") %>% 
  html_node("a") %>% 
  html_attr("href") %>% 
  paste0("http://www.imdb.com",.)

# write a function to fold the chunky scraping template
template <- function(url){
  title <- 
    url %>% 
    read_html() %>% 
    html_nodes("h1") %>% `[[`(2) %>% 
    html_text() %>% 
    stri_trim_both() %>% substr(.,1,nchar(.)-7)
  
  rating <- 
    url %>% 
    read_html() %>%
    html_node(".ratingValue") %>%
    html_text() %>%
    stri_replace_all_fixed("/10","") %>%
    stri_trim_both() %>%
    as.numeric()
  
  director <- 
    url %>% 
    read_html() %>%
    html_node(".summary_text+ .credit_summary_item") %>%
    html_text() 
  director %<>% str_split(":|,")
  director <- director[[1]][2] %>% str_trim
  
  one_movie <- data.frame(title,rating,director)
  one_movie
}

# test template 
template(link_rvest[1])

# An empty object to store data
movie_data <- NULL

# Loop through the 250 movie
# This might take a while
for (i in 1:250) {
  movie <- template(link_rvest[i])
  
  # Extend the length of data set after each iteration
  movie_data %<>% rbind(movie)
}
rm(movie,i)
save(movie_data,file="movie.rda")

# rating histogram
movie_data %>% 
  ggplot(aes(rating)) + geom_histogram(aes(y = (..count..)/sum(..count..)),binwidth =0.1,col="red", fill="green", alpha = .2)  + 
  theme_hc() + xlab("Rating") + ylab("Percentage") 

# top directors
movie_data %>% 
  tbl_df() %>% 
  group_by(director) %>% 
  summarize(freq = n()) %>% 
  arrange(desc(freq)) %>% 
  `[`(1:20,) %>% 
  kable() 



# Extension: Page Flipping -----------------------------------------------------------

url_flip <- c("http://www.imdb.com/search/title?genres=action&sort=user_rating,desc&title_type=feature&num_votes=25000,&pf_rd_m=A2FGELUUNOQJNL&pf_rd_p=2406822102&pf_rd_r=1HYX8S8DAZ1MCQ1CMYQE&pf_rd_s=right-6&pf_rd_t=15506&pf_rd_i=top&ref_=chttp_gnr_1",
              "http://www.imdb.com/search/title?genres=action&num_votes=25000,&pf_rd_i=top&pf_rd_m=A2FGELUUNOQJNL&pf_rd_p=2406822102&pf_rd_r=1HYX8S8DAZ1MCQ1CMYQE&pf_rd_s=right-6&pf_rd_t=15506&ref_=chttp_gnr_1&sort=user_rating,desc&start=51&title_type=feature",
              "http://www.imdb.com/search/title?genres=action&num_votes=25000,&pf_rd_i=top&pf_rd_m=A2FGELUUNOQJNL&pf_rd_p=2406822102&pf_rd_r=1HYX8S8DAZ1MCQ1CMYQE&pf_rd_s=right-6&pf_rd_t=15506&ref_=chttp_gnr_1&sort=user_rating,desc&start=101&title_type=feature")

compare_data <- NULL
for (i in 1:3) {
  temp <- 
    url_flip[i] %>%
    stri_replace("",fixed="http://www.imdb.com/search/title?") %>% 
    stri_split(fixed="&") %>% 
    unlist() %>% 
    strsplit("=") %>% 
    data.frame() %>% t() %>% 
    data.frame() %>% tbl_df()
  
  ifelse(i==1,compare_data <- temp,compare_data %<>% full_join(temp,by="X1"))
}
names(compare_data) <- c("LHS","RHS.P1","RHS.P2","RHS.P3")
compare_data %>% kable()



# Extension: crawl a table ------------------------------------------------

page_url <- "http://feedback.ebay.com/ws/eBayISAPI.dll?ViewFeedback2&userid=surfstitch&&_trksid=p2056016.l2560&rt=nc&iid=231843163105&sspagename=VIP%3Afeedback&ftab=FeedbackAsSeller"

fb_data <- page_url %>% read_html %>% 
  html_node(".frp") %>% html_node("table") %>% html_table()

fb_data %<>% `[`(3:5) %>% 
  unlist() %>% as.data.frame() %>% 
  t() %>% as.data.frame() %>% tbl_df()
names(fb_data) <- c("p1","m1","n1","p6","m6","n6","p12","m12","n12")
fb_data



# Extension: Java script generated content -------------------------------------------

# ATTENTION: install PhantomJS first, see slides

# some pages with java script generated content
url <- c("http://www.premierleague.com/en-gb/matchday/league-table.html", 
         "https://www.premierleague.com/tables?co=1&se=42&mw=-1&ha=-1")

epl_data <- NULL
for (i in 1:2) {
  # change the first line of scrape.js scrape
  # where the url is specified
  lines <- readLines("scrape.js") 
  lines[1] <- paste0("var url ='", url[i] ,"';")
  writeLines(lines, "scrape.js")
  ## let phantomjs download the website
  system("phantomjs scrape.js")
  ### use rvest as a html parser
  temp <- read_html("myfile.html") %>% html_node("table") %>% html_table(fill=TRUE) %>% 
    as.data.frame()%>% `[`(seq(1,39,2),c(2,3,11)) %>% tbl_df() 
  ### rename variables; remove rownames
  names(temp) <- c("pos","club","pts")
  row.names(temp) <- NULL
  ### some text processing
  temp %<>% mutate(pos = stri_trim_both(substr(pos,1,2)), 
                      club = substr(club,1,nchar(club)-3))
  ## bind data
  epl_data %<>% bind_rows(temp)
}

epl_data[1:20,] %>% kable(); epl_data[21:40,] %>% kable()




# Geocoding: driving distance between two locations -----------------------

# use google api; description below
# https://developers.google.com/maps/documentation/distance-matrix/intro?hl=en

# define a function that takes origin and destination as inputs, and miles driving distance as output
travel.dist <- function(origin,destination,mode="driving"){
  # construct URL
  xml.url <- paste0("http://maps.googleapis.com/maps/api/distancematrix/xml?origins=",origin,"&destinations=",destination,"&mode=",mode)
  # request URL
  xmlfile <- xmlParse(getURL(xml.url))
  # parse URL
  dist <- xmlValue(xmlChildren(xpathApply(xmlfile,"//distance")[[1]])$value)
  # output distance
  mile <- as.numeric(dist)*0.000621371192  # meter to mile
  return(mile)
}

travel.dist(origin='university+of+maryland+college+park',
            destination='florida+state+university')
# same as google map
# https://www.google.com/maps/dir/Columbia,+SC/Tallahassee,+FL/@32.2357752,-83.7782298,8z/data=!3m1!4b1!4m13!4m12!1m5!1m1!1s0x88f8a5697931d1e3:0xf32808f4b379fa96!2m2!1d-81.0348144!2d34.0007104!1m5!1m1!1s0x88ec8a5187124b53:0xebee077ad4fdb1f8!2m2!1d-84.2807329!2d30.4382559




# Interaction plots -------------------------------------------------------

# the conventional interaction plot
set.seed(12345)
data <-data.frame(x=rnorm(200),f=rep(c("level1","level2"),100))
data$y<-data$x*(as.numeric(as.factor(data$f))-1)+rnorm(200,,2)
lm.result <- lm(y ~ x*f,data)

# use the effect package
int.plot <- plot(effect(term="x:f",mod=lm.result,xlevels=20),multiline=TRUE)

# the ggplot reproduction
data$yhat <- lm.result$fitted.values
int.plot2 <- 
  data %>% 
  ggplot(aes(x=x, y=yhat)) +
  geom_line(aes(color=f)) +
  theme_minimal() 


# the confidence band plot
# first take out the estimates and covariance
estmean <- lm.result$coef[c(3,4)]
var <-vcov(lm.result)[c(3,4),c(3,4)]

# compute effect of f and its 95% c.i. at each value of x
x <- data$x
slopes <- estmean[1]+estmean[2]*x

SEs <- rep(NA, length(slopes))
for (i in 1:length(x)){
  j <- x[i]
  SEs[i] <- deltamethod (~ (x1) + (x2)*j, estmean, var)
}

upper <- slopes + 1.96*SEs
lower <- slopes - 1.96*SEs

data.plot <- data.frame(x, slopes, upper, lower) 
rm(var,SEs,estmean,i,j,lower,upper,x,slopes)

# obs with positive f effect
data.plot %>% 
  filter(lower>0) %>% 
  nrow()
# obs with insig. f effect
data.plot %>% 
  filter(lower<0 & upper>0) %>% 
  nrow()
# obs with negative f effect
data.plot %>% 
  filter(upper<0) %>% 
  nrow()

# the often seen confidence band plot
conf.plot <- 
  data.plot %>% 
  ggplot(aes(x,slopes)) +
  geom_line() + geom_line(aes(y=upper)) + geom_line(aes(y=lower)) +
  geom_hline(color="red",yintercept=0) + 
  labs(y="effect of f",x="level of x")

# "empirical" confidence band plot
conf.plot2 <- 
  data.plot %>% 
  ggplot(aes(x,slopes)) +
  geom_point() + geom_point(aes(y=upper)) + geom_point(aes(y=lower)) +
  geom_hline(color="blue",yintercept=0) + 
  labs(y="effect of f",x="level of x") + 
  theme_minimal() 

conf.plot3 <- 
  data.plot %>% 
  ggplot(aes(x,slopes)) +
  geom_rect(aes(xmin=-Inf, xmax=-0.93, ymin=-Inf, ymax=Inf),fill="grey90",alpha=0.05) +
  geom_rect(aes(xmin=0.75, xmax=Inf, ymin=-Inf, ymax=Inf),fill="beige",alpha=0.05) +
  geom_point() + geom_point(aes(y=upper)) + geom_point(aes(y=lower)) +
  geom_hline(color="blue",yintercept=0) + 
  labs(y="effect of f",x="level of x") + 
  theme_minimal() 

conf.plot4 <- 
  data.plot %>% 
  ggplot(aes(x,slopes)) +
  geom_rect(aes(xmin=-Inf, xmax=-0.93, ymin=-Inf, ymax=Inf),fill="grey90",alpha=0.05) +
  geom_rect(aes(xmin=0.75, xmax=Inf, ymin=-Inf, ymax=Inf),fill="beige",alpha=0.05) +
  geom_point() + geom_point(aes(y=upper)) + geom_point(aes(y=lower)) +
  geom_hline(color="blue",yintercept=0) + 
  labs(y="effect of f",x="level of x") + 
  annotate("text",x=2,y=0.05,label="Lower Bound of 95% C.I.",color="red") +
  annotate("text",x=2,y=1,label="Estimated Mean",color="red") +
  annotate("text",x=2,y=2.2,label="Upper Bound of 95% C.I.",color="red") +
  annotate("text",x=-1.8,y=3.5,label="Negative Effect: 58",color="black") +
  annotate("text",x=-0.1,y=3.5,label="Insignificant Effect: 107",color="black") + 
  annotate("text",x=1.8,y=3.5,label="Positive Effect: 35",color="black") +
  theme_minimal() 



# empirical confidence band plot with two moderators
set.seed(54321)
data <-data.frame(x=rnorm(200),z=rnorm(200),m=rnorm(200))
data$y<-data$x*(data$z-data$m*2)+rnorm(200,,2)
lm.result <- lm(y ~ x*z+x*m,data)

estmean <- lm.result$coef[c(2,5,6)]
var <-vcov(lm.result)[c(2,5,6),c(2,5,6)]

x <- data$x
z <- data$z
m <- data$m
slopes <- estmean[1]+estmean[2]*z+estmean[3]*m
e_slopes <- (estmean[1]+estmean[2]*z)/(-estmean[3])

SEs <- rep(NA, length(slopes))
e_lower <- rep(NA, length(slopes))
e_upper <- rep(NA, length(slopes))
fl <- function(j,k) {
  form <- sprintf("~ (x1) + (x2)*(%f)+(x3)*(%f)", k,j)
  estmean[1]+estmean[2]*k+estmean[3]*j - 1.96*deltamethod (as.formula(form), estmean, var)}
fu <- function(j,k) {
  form <- sprintf("~ (x1) + (x2)*(%f)+(x3)*(%f)", k,j)
  estmean[1]+estmean[2]*k+estmean[3]*j + 1.96*deltamethod (as.formula(form), estmean, var)}

for (i in 1:length(slopes)){
  k <- z[i]
  jei <- m[i]
  SEs[i] <- deltamethod (~ (x1) + (x2)*k+(x3)*jei, estmean, var)
  e_lower[i] <- uniroot(fl,c(-10000,10000),k=k,tol = 1e-15)$root
  e_upper[i] <- uniroot(fu,c(-10000,10000),k=k,tol = 1e-15)$root
}

upper <- slopes + 1.96*SEs
lower <- slopes - 1.96*SEs

data.plot <- data.frame(slopes,m,z,upper,lower,e_upper,e_lower,e_slopes) 
rm(list=setdiff(ls(),c("data","data.plot","int.plot","int.plot2","conf.plot","conf.plot2","conf.plot3","conf.plot4")))

# obs with positive f effect
data.plot %>% 
  filter(lower>0) %>% 
  nrow()
# obs with insig. f effect
data.plot %>% 
  filter(lower<0 & upper>0) %>% 
  nrow()
# obs with negative f effect
data.plot %>% 
  filter(upper<0) %>% 
  nrow()

two.plot <- 
  data.plot %>% 
  ggplot(aes(x=z,y=m)) +
  geom_point(aes(color=slopes))+
  scale_colour_gradientn(colours=rainbow(2))+
  labs(x="level of z",y="level of m",color="effect of x")

two.plot2 <- 
  data.plot %>% 
  ggplot(aes(x=z,y=m)) +
  geom_point(aes(color=slopes)) + 
  geom_smooth(aes(y=e_slopes)) +
  geom_smooth(aes(y=e_lower),se=FALSE,color="red") +
  geom_smooth(aes(y=e_upper),se=FALSE,color="black") +
  scale_colour_gradientn(colours=rainbow(2)) +
  labs(x="level of z",y="level of m",color="effect of x") +
  annotate("text",x=-3,y=-2,label="Black Line: Upper Bound of 95% C.I. = 0\nBlue Line: Mean Effect Size = 0\nRed Line: Lower Bound of 95% C.I. = 0",
           hjust = 0,size=4) +
  annotate("text",x=-3,y=2.5,label="80 obs with Positive and Significant Effect\n34 obs with Insignificant Effect\n86 obs with Negative and Significant Effect",
           hjust = 0,size=4) + 
  theme_minimal() 
